/**************************************************************************
 * Xameleon Floppy Driver                                                 *
 * by alman@l4os.info                                                     *
 * this code derived and adopted to Xameleo from the following source:    *
 *                                                                        *
 * FreeDOS32 Floppy Driver                                                *
 * by Salvo Isaja                                                         *
 *                                                                        *
 * Copyright (C) 2003-2005, Salvatore Isaja                               *
 *                                                                        *
 * This is "fdc.c" - Portable code for Floppy Disk Controller support     *
 *                                                                        *
 * This file is part of the FreeDOS32 Floppy Driver.                      *
 *                                                                        *
 * The FreeDOS32 Floppy Driver is free software; you can redistribute     *
 * it and/or modify it under the terms of the GNU General Public License  *
 * as published by the Free Software Foundation; either version 2 of the  *
 * License, or (at your option) any later version.                        *
 *                                                                        *
 * The FreeDOS32 Floppy Driver is distributed in the hope that it will    *
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty *
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 * GNU General Public License for more details.                           *
 *                                                                        *
 * You should have received a copy of the GNU General Public License      *
 * along with the FreeDOS32 Floppy Driver; see the file COPYING.txt;      *
 * if not, write to the Free Software Foundation, Inc.,                   *
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                *
 *                                                                        *
 * ACKNLOWLEDGEMENT                                                       *
 * This file is derived from fdc.c, floppy controller handler functions   *
 * Copyright (C) 1998  Fabian Nunez                                       *
 * You can download the original library from Cottontail OS Development   *
 * Library <http://www.0xfi.com/oslib/topx.html>, the file is FDC.ZIP     *
 * The author can be reached by email at: fabian@cs.uct.ac.za             *
 * or by airmail at: Fabian Nunez                                         *
 *                   10 Eastbrooke                                        *
 *                   Highstead Road                                       *
 *                   Rondebosch 7700                                      *
 *                   South Africa                                         *
 * Floppy formats and drive parameters table are from the Linux driver    *
 *  linux/kernel/floppy.c                                                 *
 *  Copyright (C) 1991, 1992  Linus Torvalds                              *
 *  Copyright (C) 1993, 1994  Alain Knaff                                 *
 **************************************************************************/

#include "fdc.h"
//#include "events.h"
#include <ioports.h>

#include <xam_errno.h>
#include <xam_driver.h>
#include <xam_requests.h>
#include <sys/xam_console.h>
#include <xam_blkdevice.h>
#include <l4/sigma0.h>
#include <l4/kdebug.h>
#include <l4io.h>

#undef NULL
#define NULL	0
#define WFC(c) while (c)
#define	fd32_cli()
#define	fd32_sti()
#define LOWMEM_ADDR  int /* A protected mode selector */

typedef void Fd32IntHandler(int);

static char			RcvBuffer[4096];
static L4_ThreadId_t		Owner = L4_nilthread;

extern "C" void _driver_entry (void);
L4_MsgTag_t xam_driver_registration_request(typeDeviceDriverRegistrationReq * req);
int WaitForInterrupt( int nIrq, int nTimeout );

extern inline LOWMEM_ADDR fd32_dmamem_get(unsigned size)
{
// We assume that requested page resides in a low memory. 
// Please note that this assuption is not always true. 
// In cases of restart driver in some way or start it to late the requested page 
// would be in a high memory, so it will be unaccessable for DMA controller. 
// alman

	L4_Fpage_t page = L4_Sigma0_GetAny( L4_GlobalId(48,1), 12, L4_CompleteAddressSpace );
	return L4_Address( page );
}


#define fd32_dmamem_free fd32_dosmem_free


#define MAXDRIVES 2 /* Newer controllers don't support 4 drives, just 2 */
#define SAFESEEK  5 /* Track to seek before issuing a recalibrate       */
//#define HAS2FDCS
//#define __DEBUG__

#ifdef __DEBUG__
  #define LOG_PRINTF(x) fd32_log_printf x
#else
  #define LOG_PRINTF(x)
#endif


/* This is called when fdc_logdisk is able to read sectors from the new disk. */
/* It fine-tunes format probing by reading informations in the boot sector.   */
extern int floppy_bootsector(Fdd *fdd, const FloppyFormat *formats, unsigned num_formats)
{
	return 0;
}


/* Floppy Drive Controller IO ports */
enum
{
    FDC_BPRI = 0x3F0, /* Base port of the primary controller   */
    FDC_BSEC = 0x370, /* Base port of the secondary controller */
    FDC_DOR  = 0x002, /* RW: Digital Output Register           */
    FDC_MSR  = 0x004, /* R : Main Status Register              */
    FDC_DRS  = 0x004, /* W : Data Rate Select Register         */
    FDC_DATA = 0x005, /* RW: Data Register                     */
    FDC_DIR  = 0x007, /* R : Digital Input Register            */
    FDC_CCR  = 0x007  /* W : Configuration Control Register    */
};


/* Command bytes (these are NEC765 commands + options such as MFM, etc) */
enum
{
    CMD_SPECIFY = 0x03, /* Specify drive timings   */
    CMD_WRITE   = 0xC5, /* Write data (+ MT,MFM)   */
    CMD_READ    = 0xE6, /* Read data (+ MT,MFM,SK) */
    CMD_SENSEI  = 0x08, /* Sense interrupt status  */
    //CMD_FORMAT  = 0x4D, /* Format track (+ MFM)    */
    CMD_READID  = 0x4A, /* Read sector Id (+ MFM)  */
    CMD_RECAL   = 0x07, /* Recalibrate             */
    CMD_SEEK    = 0x0F, /* Seek track              */
    CMD_VERSION = 0x10  /* Get FDC version         */
};


/* Bits for Fdd.flags */
enum
{
    DF_CHANGED = 1 << 0, /* Disk has been changed during the last command */
    DF_SPINUP  = 1 << 1, /* Motor spinup time elapsed, ready to transfer  */
    DF_SPINDN  = 1 << 2  /* Motor spindown time started                   */
};


/* Status structure for a Floppy Disk Controller */
typedef struct Fdc
{
    unsigned		base_port;        /* Base port for this controller: XXX0h      */
    L4_Word8_t		result[7];        /* Stores the result bytes returned          */
    L4_Word8_t		result_size;      /* Number of result bytes returned           */
    L4_Word8_t		sr0;              /* Status Register 0 after a sense interrupt */
    L4_Word8_t		dor;              /* Reflects the Digital Output Register      */
    Fdd				drive[4];		  /* Drives connected to this controller       */
}
Fdc;


/* All Floppy Drive Controllers issue an IRQ6 to report command completion. */
/* The irq_signaled variable let us know when IRQ6 happened.                */
/* Since it is a global variable, this makes the driver not reentrant, thus */
/* we use a simple busy flag to protect the critical sections (all... :-)   */
/* For the same reason, we can use just a single, shared, DMA buffer.       */
//static IrqSave      old_irq6;         /* Saves the old IRQ6 handler   */
/* TODO: At present we have no means to save the old IRQ6 handler */
//static LOWMEM_ADDR  dma_sel;          /* Selector/address of DMA buf  */
static L4_Word32_t  dma_addr;         /* Physical address of DMA buf  */
static Fdc          pri_fdc;          /* Status of the primary FDC    */
#ifdef HAS2FDCS
static Fdc          sec_fdc;          /* Status of the secondary FDC  */
#endif


/* Geometry and other format specifications for floppy disks */
static const FloppyFormat floppy_formats[32] =
{
  /*  SIZE SPT HD TRK STR GAP3  RATE SRHUT GAP3F  NAME          NR DESCRIPTION   */
    {    0,  0, 0,  0, 0, 0x00, 0x00, 0x00, 0x00, 0       }, /*  0 no testing    */
    {  720,  9, 2, 40, 0, 0x2A, 0x02, 0xDF, 0x50, "d360"  }, /*  1 360KB PC      */
    { 2400, 15, 2, 80, 0, 0x1B, 0x00, 0xDF, 0x54, "h1200" }, /*  2 1.2MB AT      */
    {  720,  9, 1, 80, 0, 0x2A, 0x02, 0xDF, 0x50, "D360"  }, /*  3 360KB SS 3.5" */
    { 1440,  9, 2, 80, 0, 0x2A, 0x02, 0xDF, 0x50, "D720"  }, /*  4 720KB 3.5"    */
    {  720,  9, 2, 40, 1, 0x23, 0x01, 0xDF, 0x50, "h360"  }, /*  5 360KB AT      */
    { 1440,  9, 2, 80, 0, 0x23, 0x01, 0xDF, 0x50, "h720"  }, /*  6 720KB AT      */
//    { 2880, 18, 2, 80, 0, 0x1B, 0x00, 0xDF, 0x6C, "H1440" }, /*  7 1.44MB 3.5"   */
    { 2880, 18, 2, 80, 0, 0x1B, 0x00, 0xCF, 0x6C, "H1440" }, /*  7 1.44MB 3.5"   */
    { 5760, 36, 2, 80, 0, 0x1B, 0x43, 0xAF, 0x54, "E2880" }, /*  8 2.88MB 3.5"   */
    { 6240, 39, 2, 80, 0, 0x1B, 0x43, 0xAF, 0x28, "E3120" }, /*  9 3.12MB 3.5"   */

    { 2880, 18, 2, 80, 0, 0x25, 0x00, 0xDF, 0x02, "h1440" }, /* 10 1.44MB 5.25"  */
    { 3360, 21, 2, 80, 0, 0x1C, 0x00, 0xCF, 0x0C, "H1680" }, /* 11 1.68MB 3.5"   */
    {  820, 10, 2, 41, 1, 0x25, 0x01, 0xDF, 0x2E, "h410"  }, /* 12 410KB 5.25"   */
    { 1640, 10, 2, 82, 0, 0x25, 0x02, 0xDF, 0x2E, "H820"  }, /* 13 820KB 3.5"    */
    { 2952, 18, 2, 82, 0, 0x25, 0x00, 0xDF, 0x02, "h1476" }, /* 14 1.48MB 5.25"  */
    { 3444, 21, 2, 82, 0, 0x25, 0x00, 0xDF, 0x0C, "H1722" }, /* 15 1.72MB 3.5"   */
    {  840, 10, 2, 42, 1, 0x25, 0x01, 0xDF, 0x2E, "h420"  }, /* 16 420KB 5.25"   */
    { 1660, 10, 2, 83, 0, 0x25, 0x02, 0xDF, 0x2E, "H830"  }, /* 17 830KB 3.5"    */
    { 2988, 18, 2, 83, 0, 0x25, 0x00, 0xDF, 0x02, "h1494" }, /* 18 1.49MB 5.25"  */
    { 3486, 21, 2, 83, 0, 0x25, 0x00, 0xDF, 0x0C, "H1743" }, /* 19 1.74MB 3.5"   */

    { 1760, 11, 2, 80, 0, 0x1C, 0x09, 0xCF, 0x00, "h880"  }, /* 20 880KB 5.25"   */
    { 2080, 13, 2, 80, 0, 0x1C, 0x01, 0xCF, 0x00, "D1040" }, /* 21 1.04MB 3.5"   */
    { 2240, 14, 2, 80, 0, 0x1C, 0x19, 0xCF, 0x00, "D1120" }, /* 22 1.12MB 3.5"   */
    { 3200, 20, 2, 80, 0, 0x1C, 0x20, 0xCF, 0x2C, "h1600" }, /* 23 1.6MB 5.25"   */
    { 3520, 22, 2, 80, 0, 0x1C, 0x08, 0xCF, 0x2e, "H1760" }, /* 24 1.76MB 3.5"   */
    { 3840, 24, 2, 80, 0, 0x1C, 0x20, 0xCF, 0x00, "H1920" }, /* 25 1.92MB 3.5"   */
    { 6400, 40, 2, 80, 0, 0x25, 0x5B, 0xCF, 0x00, "E3200" }, /* 26 3.20MB 3.5"   */
    { 7040, 44, 2, 80, 0, 0x25, 0x5B, 0xCF, 0x00, "E3520" }, /* 27 3.52MB 3.5"   */
    { 7680, 48, 2, 80, 0, 0x25, 0x63, 0xCF, 0x00, "E3840" }, /* 28 3.84MB 3.5"   */

    { 3680, 23, 2, 80, 0, 0x1C, 0x10, 0xCF, 0x00, "H1840" }, /* 29 1.84MB 3.5"   */
    { 1600, 10, 2, 80, 0, 0x25, 0x02, 0xDF, 0x2E, "D800"  }, /* 30 800KB 3.5"    */
    { 3200, 20, 2, 80, 0, 0x1C, 0x00, 0xCF, 0x2C, "H1600" }  /* 31 1.6MB 3.5"    */
};


/* Parameters to manage a floppy disk drive.                                */
/* Head load time is 16 ms for all drives except 2880 KiB, that have 15 ms. */
#define MS 1 //000 /* From ms to us */
static const DriveParams default_drive_params[] =
{
   /* T HLT  spin_up  spin_dn  sel_d  int_tmt  Autodetect formats        Nat  Name                     */
    { 0,  0, 1000*MS, 3000*MS, 20*MS, 3000*MS, { 7, 4, 8, 2, 1, 5, 3,10 }, 0, "unknown"                },
    { 1,  4, 1000*MS, 3000*MS, 20*MS, 3000*MS, { 1, 0, 0, 0, 0, 0, 0, 0 }, 1, "5.25\" DD, 360 KiB"     },
    { 2,  8,  400*MS, 3000*MS, 20*MS, 3000*MS, { 2, 5, 6,23,10,20,12, 0 }, 2, "5.25\" HD, 1200 KiB"    },
    { 3,  4, 1000*MS, 3000*MS, 20*MS, 3000*MS, { 4,22,21,30, 3, 0, 0, 0 }, 4, "3.5\" DD, 720 KiB"      },
    { 4,  8/*1*/,  400*MS, 3000*MS, 20*MS, 1500*MS, { 7, 4,25,22,31,21,29,11 }, 7, "3.5\" HD, 1440 KiB"     },
    { 5, 15,  400*MS, 3000*MS, 20*MS, 3000*MS, { 7, 8, 4,25,28,22,31,21 }, 8, "3.5\" ED, 2880 KiB AMI" },
    { 6, 15,  400*MS, 3000*MS, 20*MS, 3000*MS, { 7, 8, 4,25,28,22,31,21 }, 8, "3.5\" ED, 2880 KiB"     }
};


/* Constants for DMA transfers */
enum
{
    DMA_CHANNEL = 2,    /* Number of the DMA channel for floppy transfer */
    DMA_PAGE    = 0x81, /* Page register of that DMA channel             */
    DMA_OFFSET  = 0x04, /* Offset register of that DMA channel           */
    DMA_LENGTH  = 0x05  /* Length register of that DMA channel           */
};


/* Sets up a DMA trasfer between a device and memory.              */
/* If 'read' is TRUE, then transfer will be from memory to device, */
/* else from the device to memory.                                 */
static void dma_xfer(L4_Word32_t physaddr, unsigned length, int read)
{
    L4_Word32_t page, offset;

    /* Calculate DMA page and offset */
    page    = physaddr >> 16;
    offset  = physaddr & 0xffff;
    length -= 1; /* with DMA, if you want k bytes, you ask for k - 1 */

    fd32_cli();                                          /* Disable IRQs            */
    outb(0x0A, DMA_CHANNEL | 4);                    /* Set channel mask bit    */
    outb(0x0C, 0);                                  /* Clear flip flop         */
    outb(0x0B, (read ? 0x48 : 0x44) + DMA_CHANNEL); /* Mode (write+single+r/w) */
    outb(DMA_PAGE, page);                           /* Page                    */
    outb(DMA_OFFSET, offset & 0xFF);                /* Offset: low byte        */
    outb(DMA_OFFSET, offset >> 8);                  /* Offset: high byte       */
    outb(DMA_LENGTH, length & 0xFF);                /* Length: low byte        */
    outb(DMA_LENGTH, length >> 8);                  /* Length: high byte       */
    outb(0x0A, DMA_CHANNEL);                        /* Clear DMA mask bit      */
    fd32_sti();                                          /* enable IRQs             */
}


/* Sends a byte to the controller FIFO. */
/* Returns FDC_TIMEOUT on timeout.      */
static int sendbyte(unsigned base_port, unsigned byte)
{
    volatile unsigned msr;
    unsigned          tmo;

    for (tmo = 0; tmo < 128; tmo++)
    {
        msr = inb(base_port + FDC_MSR);
        if ((msr & 0xC0) == 0x80)
        {
            outb(base_port + FDC_DATA, byte);
            return FDC_OK;
        }
        inb(0x80); /* delay */
    }
    return FDC_TIMEOUT; /* write timeout */
}


/* Gets a byte from the controller FIFO. */
/* Returns FDC_TIMEOUT on timeout.       */
static int getbyte(unsigned base_port)
{
    volatile unsigned msr;
    unsigned          tmo;

    for (tmo = 0; tmo < 128; tmo++)
    {
        msr = inb(base_port + FDC_MSR);
        if ((msr & 0xD0) == 0xD0) return inb(base_port + FDC_DATA);
        inb(0x80); /* delay */
    }
    return FDC_TIMEOUT; /* read timeout */
}


/* Waits for FDC command to complete. Returns FDC_TIMEOUT on time out. */
static int wait_fdc(Fdd *fdd)
{
	int irq_timeout = WaitForInterrupt( 6, default_drive_params[0].int_tmout );
	outb(0x20, 0x20);

    /* Read in command result bytes while controller is busy */
    fdd->fdc->result_size = 0;
    while ((fdd->fdc->result_size < 7) && (inb(fdd->fdc->base_port + FDC_MSR) & 0x10))
        fdd->fdc->result[fdd->fdc->result_size++] = getbyte(fdd->fdc->base_port);

    if (irq_timeout)
    {
        LOG_PRINTF(("[FDC] Timeout!\n"));
        return FDC_TIMEOUT;
    }
    #if 0
    {
        unsigned i;
        LOG_PRINTF(("Result bytes: "));
        for (i = 0; i < fdd->fdc->result_size; i++)
            LOG_PRINTF(("%u(%02xh) ", fdd->fdc->result[i], fdd->fdc->result[i]));
        LOG_PRINTF(("\n"));
    }
    #endif
    return FDC_OK;
}


/* Returns nonzero if the motor of the specified drive is on */
static inline int is_motor_on(Fdc *fdc, unsigned drive)
{
    return fdc->dor & (1 << (drive + 4));
}


/* Turns the motor off after the spindown time */
static void motor_off_cb(void *params)
{
    Fdd *fdd = (Fdd *) params;
    if (is_motor_on(fdd->fdc, fdd->number))
    {
        fdd->fdc->dor &= ~(1 << (fdd->number + 4));
        outb(fdd->fdc->base_port + FDC_DOR, fdd->fdc->dor);
        fdd->flags &= ~(DF_SPINUP | DF_SPINDN);
//	xam_printf( 0, "FLOPPY: Motor off\n" );
    }
}


/* Signals that motor spinup time elapsed */
static void motor_spin_cb(void *fdd)
{
    ((Fdd *) fdd)->flags |= DF_SPINUP;
}


/* Turns the motor on and selects the drive */
static void motor_on(Fdd *fdd)
{
//    if (fdd->flags & DF_SPINDN) fd32_event_delete(fdd->spin_down);
    fdd->flags   &= ~DF_SPINDN;
    if (!is_motor_on(fdd->fdc, fdd->number))
    {
        if (!(fdd->flags & DF_SPINUP)) motor_spin_cb(fdd);
            /* TODO: Check for FD32_EVENT_NULL */
//           if (fd32_event_post(fdd->dp->spin_up, motor_spin_cb, fdd) < 0)
//                LOG_PRINTF(("Motor on: Out of events!\n"));
        fdd->fdc->dor |= (1 << (fdd->number + 4));
        /* Select drive */
        fdd->fdc->dor = (fdd->fdc->dor & 0xFC) | fdd->number;
        outb(fdd->fdc->base_port + FDC_DOR, fdd->fdc->dor);
    }
}


/* Schedule motor off setting up the spindown timer */
static void motor_down(Fdd *fdd)
{
    if (is_motor_on(fdd->fdc, fdd->number))
    {
        if (fdd->flags & DF_SPINDN) return;
//        fdd->spin_down = fd32_event_post(fdd->dp->spin_down, motor_off_cb, fdd);
        if (fdd->spin_down == -1) LOG_PRINTF(("[FDC] motor_down: out of events!\n"));
        fdd->flags |= DF_SPINDN;
    }
}


/* Returns nonzero if there was a disk change */
int fdc_disk_changed(Fdd *fdd)
{
    unsigned changeline;
    motor_on(fdd);
    changeline = inb(fdd->fdc->base_port + FDC_DIR) & 0x80;
    motor_down(fdd);
    return changeline || (fdd->flags & DF_CHANGED);
}


/* Recalibrates a drive (seek head to track 0).                     */
/* Since the RECALIBRATE command sends up to 79 pulses to the head, */
/* this function issues as many RECALIBRATE as needed.              */
/* The drive is supposed to be selected (motor on).                 */
static void recalibrate(Fdd *fdd)
{
    unsigned k;
    LOG_PRINTF(("Recalibrate\n"));
    for (k = 0; k < 13; k++)
    {
        sendbyte(fdd->fdc->base_port, CMD_RECAL);
        sendbyte(fdd->fdc->base_port, fdd->number);
        wait_fdc(fdd);
        /* Send a "sense interrupt status" command */
        sendbyte(fdd->fdc->base_port, CMD_SENSEI);
        fdd->fdc->sr0 = getbyte(fdd->fdc->base_port);
        fdd->track    = getbyte(fdd->fdc->base_port);
        if (!(fdd->fdc->sr0 & 0x10)) break; /* Exit if Unit Check is not set */
    }
    LOG_PRINTF(("Calibration result on drive %u: SR0=%02xh, Track=%u\n",
                fdd->number, fdd->fdc->sr0, fdd->track));
}


/* Seeks a drive to the specified track.            */
/* The drive is supposed to be selected (motor on). */
int fdc_seek(Fdd *fdd, unsigned track)
{
    if (fdd->track == track) return FDC_OK; /* Already there */
    if (track >= fdd->fmt->tracks) return FDC_ERROR; /* Invalid track */
    sendbyte(fdd->fdc->base_port, CMD_SEEK);
    sendbyte(fdd->fdc->base_port, fdd->number);
    sendbyte(fdd->fdc->base_port, track);
    if (wait_fdc(fdd)) return FDC_TIMEOUT; /* Timeout */
    /* Send a "sense interrupt status" command */
    sendbyte(fdd->fdc->base_port, CMD_SENSEI);
    fdd->fdc->sr0 = getbyte(fdd->fdc->base_port);
    fdd->track    = getbyte(fdd->fdc->base_port);
    /* Check that seek worked */
    if ((fdd->fdc->sr0 != 0x20 + fdd->number) || (fdd->track != track))
    {
        LOG_PRINTF(("Seek error on drive %u: SR0=%02xh, Track=%u Expected=%u\n",
                    fdd->number, fdd->fdc->sr0, fdd->track, track));
        return FDC_ERROR;
    }
    LOG_PRINTF(("Seek result on drive %u: SR0=%02xh, Track=%u\n", fdd->number,
                fdd->fdc->sr0, fdd->track));
    return FDC_OK;
}


/* Gets the position of the heads by reading the next sector identifier. */
/* Fills chs with the current head position.                             */
/* The drive is supposed to be selected (motor on).                      */
static int readid(Fdd *fdd, Chs *chs)
{
    int res = 0;
    sendbyte(fdd->fdc->base_port, CMD_READID);
    sendbyte(fdd->fdc->base_port, fdd->number);
    if (wait_fdc(fdd)) res = FDC_TIMEOUT;
    if (res < 0) return res;
    if ((fdd->fdc->result[0] & 0xC0) != 0) return FDC_ERROR;
    chs->c = fdd->fdc->result[3];
    chs->h = fdd->fdc->result[4];
    chs->s = fdd->fdc->result[5];
    return FDC_OK;
}


/* Program data rate and specify drive timings using the SPECIFY command */
static void specify(const Fdd *fdd)
{
    outb(fdd->fdc->base_port + FDC_CCR, fdd->fmt->rate);
    sendbyte(fdd->fdc->base_port, CMD_SPECIFY);
    sendbyte(fdd->fdc->base_port, fdd->fmt->sr_hut);
    sendbyte(fdd->fdc->base_port, fdd->dp->hlt << 1); /* Always DMA */
}



/* Resets the FDC to a known state */
static void reset_fdc(Fdc *fdc)
{
    unsigned      k;

    outb(fdc->base_port + FDC_DOR, 0);    /* Stop the motor and disable IRQ/DMA  */
    /* TODO: Add a small delay (20 us) to make older controllers more happy */
    outb(fdc->base_port + FDC_DOR, 0x0C); /* Re-enable IRQ/DMA and release reset */
    fdc->dor = 0x0C;

	int nStatus = WaitForInterrupt( 6, default_drive_params[0].int_tmout );
	outb(0x20, 0x20);
	switch (nStatus)
	{
	case 0:
		break;
	case 1:
        xam_printf( 0, "Timed out while waiting for FDC after reset\n");
		break;
	default:
        xam_printf( 0, "Interrupt system call aborted\n");
		break;
	}

	/* FDC specs say to sense interrupt status four times */
    for (k = 0; k < 4; k++)
    {
        /* Send a "sense interrupt status" command */
        sendbyte(fdc->base_port, CMD_SENSEI);
        fdc->sr0            = getbyte(fdc->base_port);
        fdc->drive[k].track = getbyte(fdc->base_port);
    }
}

    
/* Get a drive to a known state */
static void reset_drive(Fdd *fdd)
{
    if (fdd->dp->cmos_type == 0) return;
//    if (fdd->flags & DF_SPINDN) fd32_event_delete(fdd->spin_down);
    fdd->flags   = 0;
    fdd->track   = 0;
    specify(fdd);
    motor_on(fdd);
    fdc_seek(fdd, SAFESEEK);
    recalibrate(fdd);
    motor_down(fdd);
    fdd->flags |= DF_CHANGED; /* So that fdc_log_disk can be issued */
}


/* Transfer sectors between the disk and the DMA buffer. */
/* The drive is supposed to be selected (motor on).      */
static int fdc_xfer(Fdd *fdd, const Chs *chs, L4_Word32_t dma_address, unsigned num_sectors, FdcTransfer op)
{
//    xam_printf(0, "[FDC] fdc_xfer: C=%u, H=%u, S=%u, n=%u, op=%u\n", chs->c, chs->h, chs->s, num_sectors, op);
    /* Wait for motor spin quickly enough */
    WFC(!(fdd->flags & DF_SPINUP));

    /* Send read/write command */
    if (op == FDC_READ)
    {
        dma_xfer(dma_address, 512 * num_sectors, 0); /* Read */
        sendbyte(fdd->fdc->base_port, CMD_READ);
    }
    else
    {
        dma_xfer(dma_address, 512 * num_sectors, 1); /* Write */
        sendbyte(fdd->fdc->base_port, CMD_WRITE);
    }
    sendbyte(fdd->fdc->base_port, (chs->h << 2) | fdd->number);
    sendbyte(fdd->fdc->base_port, chs->c);
    sendbyte(fdd->fdc->base_port, chs->h);
    sendbyte(fdd->fdc->base_port, chs->s);
    sendbyte(fdd->fdc->base_port, 2); /* 512 bytes per sector */
    sendbyte(fdd->fdc->base_port, fdd->fmt->sec_per_trk);
    sendbyte(fdd->fdc->base_port, fdd->fmt->gap3);
    sendbyte(fdd->fdc->base_port, 0xFF); /* DTL (bytes to transfer) = unused */

    /* Wait for completion and produce exit code */
    if (wait_fdc(fdd))
    {
        reset_fdc(fdd->fdc);
        reset_drive(fdd);
        return FDC_TIMEOUT;
    }
    if ((fdd->fdc->result[0] & 0xC0) == 0) return FDC_OK;
    return FDC_ERROR;
}


/* Reads sectors from the floppy, up to the end of the cylinder.         */
/* If buffer is NULL, the read data is discarded (may be used to probe). */
int fdc_read(Fdd *fdd, const Chs *chs, L4_Word8_t *buffer, unsigned num_sectors)
{
    unsigned tries;
    int      res = FDC_ERROR;

    if (fdd->dp->cmos_type == 0) return FDC_ERROR; /* Drive not available */
    /* TODO: Place a timeout for Busy check */
    motor_on(fdd);
    specify(fdd);
    for (tries = 0; tries < 3; tries++)
    {
        /* Move head to right track */
        if (fdc_seek(fdd, chs->c) == FDC_OK)
        {
            /* If changeline is active, no disk is in drive */
            if (inb(fdd->fdc->base_port + FDC_DIR) & 0x80)
            {
                xam_printf(0, "[FDC] fdc_read: no disk in drive\n");
                res = FDC_NODISK;
                break;
            }
            res = fdc_xfer(fdd, chs, dma_addr, num_sectors, FDC_READ);
            if (res == FDC_OK) break;
        }
        else res = FDC_ERROR; /* Seek error */
        if (tries != 2) recalibrate(fdd); /* Error, try again... */
    }
    motor_down(fdd);
    return res;
}


/* Writes sectors to the floppy, up to the end of the cylinder. */
int fdc_write(Fdd *fdd, const Chs *chs, const L4_Word8_t *buffer, unsigned num_sectors)
{
    unsigned tries;
    int      res = FDC_ERROR;

    if (fdd->dp->cmos_type == 0) return FDC_ERROR; /* Drive not available */
    /* TODO: Place a timeout for Busy check */
    motor_on(fdd);
    specify(fdd);
    for (tries = 0; tries < 3; tries++)
    {
        /* Move head to right track */
        if (fdc_seek(fdd, chs->c) == FDC_OK)
        {
            /* If changeline is active, no disk is in drive */
            if (inb(fdd->fdc->base_port + FDC_DIR) & 0x80)
            {
                xam_printf(0, "[FDC] fdc_write: no disk in drive\n");
                res = FDC_NODISK;
                break;
            }
            res = fdc_xfer(fdd, chs, dma_addr, num_sectors, FDC_WRITE);
            if (res == FDC_OK) break;
        }
        else res = FDC_ERROR; /* Seek error */
        if (tries != 2) recalibrate(fdd); /* Error, try again... */
    }
    motor_down(fdd);
    return res;
}


/* Transfers data between a whole cylinder (both tracks of disk sides) and */
/* and the DMA buffer, as fast as possible.                                */
/* We get the current head position with READID, so we can start to        */
/* transfer sectors from the first sector passing below the head. When the */
/* disk completes a revolution, we finish transferring the first sectors.  */
/* If buffer is NULL, the read data is discarded (may be used to probe).   */
static int fdc_xfer_cylinder(Fdd *fdd, unsigned cyl, FdcTransfer op)
{
    unsigned tries;               /* Count how many retries              */
    Chs      chs = { cyl, 0, 1 }; /* Passed to fdc_xfer to read sectors  */
    Chs      cur;                 /* Current position returned by READID */
    unsigned sec_in_cyl;          /* Zero-based sector we start to read  */
    int      res = FDC_ERROR;     /* This function's result...           */

    motor_on(fdd);
    specify(fdd);
    for (tries = 0; tries < 3; tries++)
    {
        /* Move head to right track */
        if (fdc_seek(fdd, cyl) == FDC_OK)
        {
            /* If changeline is active, no disk is in drive */
            if (inb(fdd->fdc->base_port + FDC_DIR) & 0x80)
            {
                LOG_PRINTF(("[FDC] fdc_Xfer_cylinder: no disk in drive\n"));
                res = FDC_NODISK;
                break;
            }
            readid(fdd, &cur);
            LOG_PRINTF(("[FDC] fdc_xfer_cylinder: readid=%u,%u,%u\n", cur.c, cur.h, cur.s));
            /* Transfer from the current sector of head 0 to the end of cylinder */
            cur.s++; /* The sector we read the Id is gone. Advance a bit */
            sec_in_cyl = cur.s - 1;
            if (cur.s > fdd->fmt->sec_per_trk)
            {
                cur.h++;
                cur.s -= fdd->fmt->sec_per_trk;
            }
            chs.h = cur.h;
            chs.s = cur.s;
            LOG_PRINTF(("[FDC] fdc_xfer_cylinder: first round:%u sectors from %u\n",
                        fdd->fmt->sec_per_trk * 2 - sec_in_cyl, sec_in_cyl));
            res = fdc_xfer(fdd, &chs, dma_addr + 512 * sec_in_cyl,
                           fdd->fmt->sec_per_trk * 2 - sec_in_cyl, op);
            /* Read the first sectors, that we missed in the previous revolution */
            if ((res == FDC_OK) && (sec_in_cyl > 0))
            {
                chs.h = 0;
                chs.s = 1;
                LOG_PRINTF(("[FDC] fdc_read_cylinder: second round:%u sectors from 0\n", sec_in_cyl));
                res = fdc_xfer(fdd, &chs, dma_addr, sec_in_cyl, op);
            }
            if (res == FDC_OK) break;
        }
        else res = FDC_ERROR; /* Seek error */
        if (tries != 2) recalibrate(fdd); /* Error, try again... */
    }
    motor_down(fdd);
    return res;
}


/* Reads a whole cylinder (both tracks of disk sides) as fast as possible. */
/* See fdc_xfer_cylinder for comments.                                     */
/* If buffer is NULL, the read data is discarded (may be used to probe).   */
int fdc_read_cylinder(Fdd *fdd, unsigned cyl, L4_Word8_t *buffer)
{
    int res;
    if (fdd->dp->cmos_type == 0) return FDC_ERROR; /* Drive not available */
    /* TODO: Place a timeout for Busy check */
    res = fdc_xfer_cylinder(fdd, cyl, FDC_READ);
    return res;
}


/* Writes a whole cylinder (both tracks of disk sides) as fast as possible. */
/* See fdc_read_cylinder for comments.                                      */
int fdc_write_cylinder(Fdd *fdd, unsigned cyl, const L4_Word8_t *buffer)
{
    int res;
    if (fdd->dp->cmos_type == 0) return FDC_ERROR; /* Drive not available */
    /* TODO: Place a timeout for Busy check */
    res = fdc_xfer_cylinder(fdd, cyl, FDC_WRITE);
    return res;
}


/* Ckecks for the specified disk format, by reading the last sector of the */
/* last track of the last side of the specified format.                    */
/* Returns zero if format is supported, or a negative value if not.        */
static int probe_format(Fdd *fdd, unsigned format)
{
    Chs chs = { 0, 0, 0 };
    int res;

    fdd->fmt = &floppy_formats[format];
    chs.s = fdd->fmt->sec_per_trk;
    res = fdc_read(fdd, &chs, NULL, 1);
//    xam_printf(0, "[FDC] probe_format: fdc_read returned %i\n", res );
    if (res < 0) return res;
//    xam_printf(0, "%s format detected\n", fdd->fmt->name);
    return FDC_OK;
}


/* Checks disk geometry. Call this after any disk change. */
/* Returns 0 on success.                                  */
int fdc_log_disk(Fdd *fdd)
{
    unsigned k;
    if (fdd->dp->cmos_type == 0) return FDC_ERROR; /* Drive not available */
    fdd->fmt = &floppy_formats[fdd->dp->native_fmt];
    reset_fdc(fdd->fdc);
    reset_drive(fdd);
    /* If changeline is active, no disk is in drive */
    motor_on(fdd);
    if (inb(fdd->fdc->base_port + FDC_DIR) & 0x80)
    {
        xam_printf(0, "[FDC] fdc_log_disk: no disk in drive\n");
        motor_down(fdd);
        return FDC_NODISK;
    }
    fdd->flags &= ~DF_CHANGED;
    for (k = 0; k < 8; k++)
        if (probe_format(fdd, fdd->dp->detect[k]) == 0)
        {
            /* Finetune the probed format with boot sector informations */
            //int res = floppy_bootsector(fdd, floppy_formats, 32);
            //if (res >= 0) fdd->fmt = &floppy_formats[res];
            return FDC_OK;
        }
    motor_down(fdd);
    fdd->flags |= DF_CHANGED; /* Must log the disk again the next time */
    return FDC_ERROR; /* If we arrive here, the autodetection failed */
}


/* Initializes a drive to a known state */
static int setup_drive(Fdc *fdc, unsigned drive, unsigned cmos_type)
{
    fdc->drive[drive].fdc       = fdc;
    fdc->drive[drive].dp        = &default_drive_params[0];
    /* No need to initialize fdc->drive[drive].fmt */
    fdc->drive[drive].number    = drive;
    fdc->drive[drive].flags     = 0;
    fdc->drive[drive].track     = 0;
    fdc->drive[drive].spin_down = -1;
    if ((cmos_type > 0) && (cmos_type <= 6))
    {
//        xam_printf( 0, "[FLOPPY] Drive %u set to %s\n", drive, default_drive_params[cmos_type].name);
        fdc->drive[drive].dp        = &default_drive_params[cmos_type];
        fdc->drive[drive].flags     = DF_CHANGED;
    }
    return FDC_OK;
}


/* Initializes the low-level driver */
int fdc_setup(FdcSetupCallback *setup_cb)
{
    unsigned		k;
    unsigned		cmos_drive0;
    unsigned		cmos_drive1;
    int				res;

    /* Setup IRQ and DMA */
//	L4_AssociateInterrupt( L4_GlobalId (6, 1), L4_GlobalIdOf(L4_Myself()) );
    /* TODO: Find a decent size for the DMA buffer */
    dma_addr = fd32_dmamem_get( 512 * 18 * 2 );
//    xam_printf(0, "DMA buffer allocated at physical address %08lxh\n", dma_addr);


    /* Reset primary controller */
    pri_fdc.base_port = FDC_BPRI;
    reset_fdc(&pri_fdc);
    sendbyte(pri_fdc.base_port, CMD_VERSION);
    res = getbyte(pri_fdc.base_port);
//    xam_printf(0, "Byte got from CMD_VERSION: %08xh\n", res);
/*
    switch (res)
    {
	case 0x80: xam_printf( 0, "FLOPPY: NEC765 FDC found on base port %04xh\n", pri_fdc.base_port); break;
	case 0x90: xam_printf( 0, "FLOPPY: Enhanced FDC found on base port %04xh\n", pri_fdc.base_port); break;
	default:   xam_printf( 0, "FLOPPY: FDC not found on base port %04xh\n", pri_fdc.base_port);
    }
*/
    
    /* Read floppy drives types from CMOS memory (up to two drives). */
    /* They are supposed to belong to the primary FDC.               */
    outb(0x70, 0x10);
    k = inb(0x71);
    cmos_drive0 = k >> 4;
    cmos_drive1 = k & 0x0F;
    setup_drive(&pri_fdc, 0, cmos_drive0);
    setup_drive(&pri_fdc, 1, cmos_drive1);

#if 0
    for (res = FDC_OK, k = 0; k < MAXDRIVES; k++)
        if (pri_fdc.drive[k].dp->cmos_type)
            if (setup_cb(&pri_fdc.drive[k]) < 0) res = -1;
#endif

    #ifdef HAS2FDCS
    setup_drive(&sec_fdc, 0, 0);
    setup_drive(&sec_fdc, 1, 0);
    sec_fdc.base_port = FDC_BSEC;
    reset_fdc(&sec_fdc);
    for (k = 0; k < MAXDRIVES; k++)
        if (setup_cb(&sec_fdc.drive[k]) < 0) res = -1;
    #endif
    return res;
}


/* Frees low-level driver resources */
void fdc_dispose(void)
{
    /* TODO: How to restore old handler? */
//    fd32_irq_bind(6, &old_irq6);
//    LOG_PRINTF(("IRQ6 handler uninstalled\n"));
    /* TODO: Find a decent size for the DMA buffer */
//    fd32_dmamem_free(dma_sel, 512 * 18 * 2);
    LOG_PRINTF(("DMA buffer freed\n"));
    outb(pri_fdc.base_port + FDC_DOR, 0x0C); /* Stop motor forcefully */
}



/* Converts an absolute sector number (LBA) to CHS using disk geometry.     */
/* Returns 0 on success, or a negative value meaning invalid sector number. */
static inline int lba_to_chs(unsigned lba, Chs *chs, const Fdd *drive)
{
    unsigned temp;
    unsigned cyl_bytes = drive->fmt->heads * drive->fmt->sec_per_trk;

    if (lba >= drive->fmt->tracks * cyl_bytes) return -1;
    chs->c = lba / cyl_bytes;
    temp   = lba % cyl_bytes;
    chs->h = temp / drive->fmt->sec_per_trk;
    chs->s = temp % drive->fmt->sec_per_trk + 1;
    return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Name       :
// Return     :                                                              
// Description:
//                                                                           
L4_ThreadId_t ReadBlock(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	int							nStatus;
	L4_Word_t					nContext;	
	L4_Word_t					nDriveNumber;
	L4_Word_t					nBlockNumber;
	L4_Word_t					nBlockSize;
//	L4_Word_t					nPosition;
	L4_Word_t					nCachingType;
	Fdd						*	pDrive;
	Chs							chs;
	L4_StringItem_t				Block;
	
	do {

		nContext		= L4_Get (msg, 0);
		nDriveNumber	= L4_Get (msg, 1);
		nBlockNumber	= L4_Get (msg, 2);
		nBlockSize		= L4_Get (msg, 3);
		nCachingType	= L4_Get (msg, 4);

        Block = L4_StringItem ( 0, (void*) 0 );

		if(L4_UntypedWords (tag) != 5) 
		{
			xam_printf(1, "floppy.drv: request 'ReadBlock' format error\n" );
			nStatus = XAM_EINVAL;
			break;
		}

//		xam_printf(1, "IDE: READ_BLOCK [%d] request %d bytes\n", nBlockNumber, nBlockSize );
			
		if(nDriveNumber >= 2)
		{
			xam_printf(1, "FLOPPY: 'ReadBlock' device is not present\n" );
			nStatus = XAM_EINVAL;
			break;
		}
		
		if( nBlockSize & (512-1) )
		{
			xam_printf(1, "FLOPPY: 'ReadBlock' block size is not sector aligned\n" );
			nStatus = XAM_EINVAL;
			break;
		}
		
		pDrive = &pri_fdc.drive[nDriveNumber];
		nBlockSize >>= 9;
		nBlockNumber *= nBlockSize;
/*		
		if( pLD->TotalSectors && nBlockNumber >= pLD->TotalSectors )
		{
			xam_printf(1, "FLOPPY: 'ReadBlock' request outside logical disk space\n" );
			Status = -1;
			break;
		}
		
		nPosition = pLD->StartingSector + nBlockNumber;
*/		
		nStatus = lba_to_chs( nBlockNumber, &chs, pDrive );
		if( nStatus ) 
		{
			xam_printf( 0, "FLOPPY: 'ReadBlock' unable translate block %d to CHS for drive %s\n", nBlockNumber, pDrive->fmt->name);
			break;
		}
		
		nStatus = fdc_read( pDrive, &chs, (L4_Word8_t*) dma_addr, nBlockSize );
		if( nStatus )
		{
			xam_printf( 0, "FLOPPY: 'ReadBlock' device read error %d\n", nStatus );
			break;
		}

		//xam_printf(1, "IDE: 'ReadBlock' dev %d, sc %d sector %d\n",
		//	pLD->DeviceNumber,
		//	nBlockSize,
		//	nPosition);

		nStatus = nBlockSize <<= 9;
        Block = L4_StringItem ( nBlockSize, (void*) dma_addr );

	} while(false);

	L4_Clear(msg);
    L4_Set_Label(msg, BlockDevice_Read);
	L4_Append(msg, nContext);		// Context
	L4_Append(msg, nStatus);
	L4_Append(msg, &Block);

	return Owner;
}

/////////////////////////////////////////////////////////////////////////////
// Name       :
// Return     :                                                              
// Description:
//                                                                           
L4_ThreadId_t WriteBlock(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	L4_Word_t					nContext;

	xam_printf(0, "FLOPPY: Write disabled. The read-only device\n" );

	L4_Get( msg, 0, &nContext );
	L4_Clear( msg );
    L4_Set_Label( msg, BlockDevice_Write );
	L4_Append( msg, nContext );
	L4_Append( msg, XAM_EROFS );

	return Owner;
}


/////////////////////////////////////////////////////////////////////////////
// Name       : GetType
// Return     :                                                              
// Description:
//                                                                           
L4_ThreadId_t Open(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	L4_ThreadId_t	Own;

	L4_Word_t	nContext = L4_Get (msg, 0);
	L4_Word_t	nMinor = L4_Get (msg, 1);
		
	Own.raw = L4_Get (msg, 2);
	
	L4_Clear(msg);
	L4_Set_Label(msg, BlockDevice_Open );
	L4_Append(msg, nContext);		// Context

	if( L4_IsNilThread( Owner ) )
	{
		L4_Append(msg, 0);			// Status
		L4_Append(msg, 0x01);			// Type FAT12
		Owner = Own;
	}
	else
	{
		L4_Append(msg, XAM_EBUSY);		// Status
		L4_Append(msg, 0);			// Type
	}
	return Owner;
}

/////////////////////////////////////////////////////////////////////////////
// Name       : Close
// Return     :                                                              
// Description:
//                                                                           
L4_ThreadId_t Close(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	L4_ThreadId_t	Own;

	L4_Word_t	nContext = L4_Get (msg, 0);
	L4_Word_t	nMinor = L4_Get (msg, 1);
		
	Own = Owner;
	
	L4_Clear(msg);
	L4_Set_Label(msg, BlockDevice_Close );
	L4_Append(msg, nContext);		// Context

	if( ! L4_IsNilThread( Owner ) )
	{
		L4_Append(msg, 0);			// Status
		Owner = L4_nilthread;
	}
	else
	{
		L4_Append(msg, XAM_EBUSY);		// Status
	}
	return Own;
}

/////////////////////////////////////////////////////////////////////////////
// Name       : FlushDevice
// Return     :                                                              
// Description:
//                                                                           
L4_ThreadId_t FlushDevice(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	L4_Word_t	nContext = L4_Get (msg, 0);
	L4_Word_t	nMinor = L4_Get (msg, 1);

	xam_printf( 3, "Floppy: Got flush request\n" );

	L4_Clear(msg);
	L4_Set_Label(msg, BlockDevice_Flush );
	L4_Append(msg, nContext);		// Context
	L4_Append(msg, 0);				// Status

	return Owner;
}

L4_ThreadId_t IOCTLDevice(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	L4_Word_t	nContext = L4_Get (msg, 0);
	L4_Word_t	nMinor = L4_Get (msg, 1);
		
	L4_Clear(msg);
	L4_Set_Label(msg, BlockDevice_IOCTL );

	L4_Append(msg, nContext);			// Context
	L4_Append(msg, XAM_ENOSYS);			// Status

	return Owner;
}

/////////////////////////////////////////////////////////////////////////////
// Name       : driver_main                                                  
// Return     :                                                              
// Description: Driver entry function                                        
//                                                                           
extern "C" void driver_main (void)
{
	xam_console_init("vty");
	fdc_setup(0);
    
	fdc_log_disk(&pri_fdc.drive[0]);
	
	xam_printf(0, "Floppy driver started for %s\n", pri_fdc.drive[0].fmt->name );

	L4_ThreadId_t				TID;
	L4_MsgTag_t					tag;
	L4_Msg_t					msg;
	L4_MsgBuffer_t				MsgBuff;
	L4_StringItem_t 			str;

	str = L4_StringItem (sizeof(RcvBuffer), (void*) RcvBuffer);

	L4_Clear(&MsgBuff);
	L4_Append(&MsgBuff, str);
	L4_Accept( L4_StringItemsAcceptor, &MsgBuff );
	
	// Server loop
	while(true)
	{
	
		tag = L4_Wait ( L4_TimePeriod( 1000*1000*10 ), &TID );
		if (L4_IpcFailed (tag))
		{
			if( L4_ErrorCode() == 0x3 ) // Timeout?
			{
				motor_off_cb( (Fdd *) &pri_fdc.drive[0] );
 			}
			else
			{
				xam_printf( 0, "FLOPPY: IPC failed : Error code 0x%lx\n", L4_ErrorCode() );
			}
			continue;
		}
		L4_Store (tag, &msg);

		switch( L4_Label(tag) & 0xff )
		{

		case BlockDevice_Open:
			TID = Open( tag, &msg );
			break;
		
		case BlockDevice_Close:
			/*TID = */ Close( tag, &msg );
			break;

		case BlockDevice_Read:
			TID = ReadBlock( tag, &msg );
			break;

		case BlockDevice_Write:
			TID = WriteBlock( tag, &msg );
			break;

		case BlockDevice_Flush:
			TID = FlushDevice( tag, &msg );
			break;

		case BlockDevice_IOCTL:
			TID = IOCTLDevice( tag, &msg );
			break;

		default:
			xam_printf (0, "FLOPPY: got unknonw msg [0x%08lx] (%p, %p) from %p\n", 
				(long) L4_Label (tag),
				(void *) L4_Get (&msg, 0), 
				(void *) L4_Get (&msg, 1),
				(void *) 0 /*TID.raw*/);
	
#define L4_REQUEST_MASK         ( ~((~0UL) >> ((sizeof (L4_Word_t) * 8) - 20)))

			xam_printf (0, "   ->[0x%08lx] Untyped words %lx Typed words %lx\n",
				tag.raw & L4_REQUEST_MASK,
				L4_UntypedWords (tag),
				L4_TypedWords (tag) );
			break;
		}
		L4_Set_Label( &msg, L4_Label(tag) | 0x8000 );
		L4_Load(&msg);
		tag = L4_Send(TID);
	}
}

extern L4_MsgTag_t xam_driver_registration_request(const typeDeviceDriverRegistrationReq * req);

#if 0

extern "C" void _driver_entry	(void);
extern "C" void _text_start		(void);
extern "C" void _text_size		(void);
extern "C" void _data_start		(void);
extern "C" void _data_size		(void);
extern "C" void __bss_start		(void);
extern "C" void __bss_size		(void);


const	typeDeviceDriverRegistrationReq		registration_request = 
{
	1,
	category_BlockDeviceDriver,
	drivermodel_Process,
	0, // Minor 
	1, // MinorCount 
	{ 
		{ (L4_Word_t) _driver_entry, 
			(L4_Word_t) _text_start, (L4_Word_t) _text_size, 
			(L4_Word_t) _data_start, (L4_Word_t) _data_size, 
			(L4_Word_t) __bss_start, (L4_Word_t) __bss_size, 
			0, 16384 } 
	},
	0, // MaxHeapSize 
	{ 'f','d',0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
	MagicLabelValue
};

#else

const	typeDeviceDriverRegistrationReq		registration_request = 
{
	1,
	category_BlockDeviceDriver,
	drivermodel_Module,
	0, // Minor 
	1, // MinorCount 
	{ 
		{ (L4_Word_t) _driver_entry, 0, 0, 0, 0, 0, 0, 0, 8192 } 
	},
	0, // MaxHeapSize 
	{ 'f','d',0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
	MagicLabelValue
};

#endif

///////////////////////////////////////////////////////////////////////////////
// Name		  : register_driver
// Return	  : Status
// Description:	Called by kernel to initiaite the driver startup request
//
extern "C" int register_driver(void)
{
	L4_ThreadId_t				tid;
	L4_MsgTag_t					tag;
	L4_Msg_t					msg;
	L4_Word_t					nStatus;

	do {

		tag = xam_driver_registration_request( &registration_request );
		if (L4_IpcFailed(tag)) 
		{
			L4_KDB_Enter( "Floppy registration IPC failed" );
			nStatus = L4_ErrorCode();
			break;
		}
		L4_Store(tag, &msg);
		nStatus = L4_Get( &msg, 0 );
		if( nStatus != 0 )
		{
			L4_KDB_Enter( "Got bad status of floppy registration" );
			break;
		}

		tid.raw =  L4_Get( &msg, 1 );
		L4_AssociateInterrupt( L4_GlobalId (6, 1), L4_GlobalIdOf( tid) );
		printf( "Floppy: interrupt associated to (%d.%d)\n", L4_ThreadNo(tid), L4_Version(tid) );

	} while( false );

	return 0;
}

