#ifndef _STRING_H_
#define _STRING_H_

//#error Hello world

#if 1

#include <sys/cdefs.h>
#include <ctype.h>
#include <sys/types.h>


	__BEGIN_DECLS

	int memcmp( const void *str1, const void *str2, unsigned int len);
	void * memcpy ( void * dst, const void * src, unsigned int len);
	void * memset ( void * dst, unsigned int c, unsigned int len);
	void * memmove( void * dst, const void * src, unsigned int len);
	void * memchr ( const void * s, int c, size_t len );
	int strlen( const char *src );
	char * strcpy( char *dst, const char *src );
	char * strcat(char * dest, const char * src);
	char * strncat(char * dest, const char * src, size_t sz);
	char * strncpy( char *dst, const char *src, unsigned long len );
	char * stpcpy( char *dst, const char *src );
	int strcmp( const char *str1, const char *str2 );
	int strncmp( const char *s1, const char *s2, int len );
	char * strrchr( const char *s1, int c );
	char * strchr( const char *s1, int c );
	char * strstr( const char * str, const char * substr);
	char * strpbrk(const char * s, const char * accept);
	char * strtok_r(char *s, const char *delim, char **last);
	char * strtok( char *s, const char *delim );
	size_t strspn(const char *s, const char * charset);
	size_t strcspn(const char * s, const char * charset);
	char * strdup( char * str);

	// Recently added functions
	int strpcmp(const char ** s1, const char ** s2);
	void *mempcpy(void *dest, const void *src, size_t n);
	const char * strerror( int errnum );
	
	// Recently added functions
	int strcasecmp( const char *str1, const char *str2 );
	int strncasecmp( const char *str1, const char *str2, size_t len );
	char *strchrnul( const char *str, int ch );
	int strcoll( const char * s1, const char * s2);

	__END_DECLS

#else

inline int memcmp(const void *str1, const void *str2, unsigned int len)
{
	unsigned int i;
    unsigned char  *s1 = (unsigned char  *) str1;
    unsigned char  *s2 = (unsigned char  *) str2;
	
    for( i = 0; i < len; i++ ) if( s1[i] != s2[i] ) return s1[i] - s2[i];
    return 0;
}

inline void * memcpy (void * dst, const void * src, unsigned int len)
{
    unsigned char  *d = (unsigned char  *) dst;
    unsigned char  *s = (unsigned char  *) src;

    while (len-- > 0) *d++ = *s++;

    return dst;
}

inline void * memset (void * dst, unsigned int c, unsigned int len)
{
    unsigned char *d = (unsigned char *) dst;

    while (len-- > 0) *d++ = c;

    return dst;
}

inline void * memmove(void * dst, const void * src, unsigned int len)
{
    unsigned char *d = (unsigned char *) dst;
    unsigned char  *s = (unsigned char  *) src;
	
	if( d < s )
    	while (len-- > 0) *d++ = *s++;
	else
    	while (len-- > 0) d[len] = s[len];
	
    return dst;
}

inline int strlen( const char *src )
{
    int cnt = 0;

    while( src[cnt] )
	cnt++;
    return cnt;
}

inline char * strcpy( char *dst, const char *src )
{
    char *d = dst;
    while( *src ) 
    {
		*d++ = *src++;
	}
	*d = *src;
    return dst;
}

inline char * strcat(char * dest, const char * src)
{
	char *d = dest;
	while( *d ) d++;
    while( *src ) 
    {
		*d++ = *src++;
	}
	*d = *src;
    return dest;
}

inline char * strncpy( char *dst, const char *src, unsigned long len )
{
    char *d = dst;
    while( *src && len-- ) 
    {
		*d++ = *src++;
    }
    return dst;
}

inline int strcmp( const char *str1, const char *str2 )
{
    while( *str1 && *str2 ) 
    {
		if( *str1 < *str2 ) return -1;
		if( *str1 > *str2 ) return 1;
		str1++;
		str2++;
    }
    if( *str2 ) return -1;
    if( *str1 )	return 1;
    return 0;
}

inline int strncmp( const char *s1, const char *s2, int len )
{
    while( len > 0 ) {
	if( !*s1 && !*s2 )
	    return 0;
	if( (!*s1 && *s2) || (*s1 < *s2) )
	    return -1;
	if( (*s1 && !*s2) || (*s1 > *s2) )
	    return 1;
	s1++;
	s2++;
	len--;
    }
    return 0;
}

inline char * strrchr(const char *s1, int c)
{
	char	*	val = 0;
	while(*s1)
	{
		if(*s1 == (const char) c)
		{
			val = (char*) s1;
		}
		s1++;
	}
	return val;
}

inline char * strchr(const char *s1, int c)
{
	char	*	val = 0;
	while(*s1)
	{
		if(*s1 == (const char) c) 
		{
			val = (char*) s1;
			break;
		}
		s1++;
	}
	return val;
}

inline char * strstr(const char * str, const char * substr)
{
	int i,j;

	while( *str ) 
	{
		j = i = 0;
		while(substr[i]) 
		{
			if(substr[i] != str[j]) break;
			i++;
			j++;
		}
		if(!substr[i]) break;
		str++; 
	}

	if(!*str) str = 0;
	return (char*) str;
}

inline char * strpbrk(const char * s, const char * accept)
{
	int	i;
	while( *s )
	{
		for( i=0; accept[i]; i++ )
		{
			if( accept[i] == *s ) return (char*) s;
		}
		s++;
	}
	return 0;
}
#endif

#endif
