#ifndef _SYS_TYPES_H_
#define _SYS_TYPES_H_

#define COUNT_OF(x)		( sizeof(x)/sizeof(x[0]) )
#define MAX_FILENAME_ZIZE	254

#include <stddef.h>

typedef struct dev_tag
{
	unsigned long		Major;		/* Must be L4 Thread ID */
	unsigned long		Minor;		/* Device number */
} rdev_t;

typedef unsigned long long	dev_t;

typedef unsigned long int		off_t;		/* offset within a file */
typedef unsigned long int		time_t;	/* time in sec since 1 Jan 1970 0000 GMT */
typedef unsigned long int		ino_t;	/* i-node number */

typedef unsigned short			mode_t;		/* file type and permissions bits */

typedef unsigned char			u_char;
typedef unsigned int			u_int;
typedef unsigned short			u_short;
typedef unsigned long			u_int32_t;
typedef unsigned int 			socklen_t;
typedef long long			int64_t;

typedef char					gid_t;		/* group id */
typedef short					uid_t;		/* user id */

typedef char					nlink_t;	/* number of links to a file */
typedef unsigned long int		zone_t;		/* zone number */
typedef unsigned long int		time_type;	/* time in sec since 1 Jan 1970 0000 GMT */

typedef int						id_t;
typedef int						pid_t;		/* Process identifier */
typedef unsigned long	typeBlockNumber;	/* disk block number */
//typedef unsigned int				size_t;		/* ssize_t */

typedef unsigned short			fsid_t;		/* File system ID */

#if 0
	typedef struct _quad { long val[2]; } quad_t;
	typedef struct _u_quad { unsigned long val[2]; } u_quad_t;
#else
	#define quad_t	long long
	#define u_quad_t unsigned long long
#endif

typedef enum		// fm_tag
{
	DoNotFlush			= 0,
	FreeIndirectCache	= 1,
	FlushInodeToDisk	= 2
} FlushMode_t;

typedef enum		DescriptorMode_t 
{ 
	NOMODE, 
	READONLY, 
	WRITEONLY, 
	READWRITE 
} DescriptorMode_t;


/*****************************************************************************
	fcntl.h
 *****************************************************************************/
#define O_NOLINK		 040000

#if 0

#if defined(_KERNEL)

#define O_ACCMODE          0003
#define O_RDONLY             00
#define O_WRONLY             01
#define O_RDWR               02
#define O_CREAT            0100 /* not fcntl */
#define O_EXCL             0200 /* not fcntl */
#define O_NOCTTY           0400 /* not fcntl */
#define O_TRUNC           01000 /* not fcntl */
#define O_APPEND          02000
#define O_NONBLOCK        04000
#define O_NDELAY        O_NONBLOCK
#define O_SYNC           010000
#define O_FSYNC          O_SYNC
#define O_ASYNC          020000
#define O_NOLINK		 040000
#define O_SYMLINK		 100000

#ifdef __USE_GNU
# define O_DIRECT        040000 /* Direct disk access.  */
# define O_DIRECTORY    0200000 /* Must be a directory.  */
# define O_NOFOLLOW     0400000 /* Do not follow links.  */
#endif

#endif

#endif
/*****************************************************************************
	sys/types.h
 *****************************************************************************/

#define SEEK_SET    0
#define SEEK_CUR    1
#define SEEK_END    2

#pragma pack(push, 1)

///////////////////////////////////////////////////////////////////////////////
// 
// File status structure
// 
// 
typedef struct stat
{
	dev_t				st_dev;			/* major/minor device number */
	ino_t				st_ino;			/* i-node number */
	unsigned long int	st_mode;		/* file mode, protection bits, etc. */
	short int			st_nlink;		/* # links; TEMPORARY HACK: should be nlink_t*/
	unsigned long int	st_uid;			/* uid of the file's owner */
	short int			st_gid;			/* gid; TEMPORARY HACK: should be gid_t */
	dev_t				st_rdev;
	off_t				st_size;		/* file size */
	time_t				st_atime;		/* time of last access */
	time_t				st_mtime;		/* time of last data modification */
	time_t				st_ctime;		/* time of last file status change */
} stat_t;

///////////////////////////////////////////////////////////////////////////////
// 
// 
// 
// 
typedef struct statfs 
{
	long    f_type;     /* type of filesystem (see below) */
	long    f_bsize;    /* optimal transfer block size */
	long    f_blocks;   /* total data blocks in file system */
	long    f_bfree;    /* free blocks in fs */
	long    f_bavail;   /* free blocks avail to non-superuser */
	long    f_files;    /* total file nodes in file system */
	long    f_ffree;    /* free file nodes in fs */
	fsid_t  f_fsid;     /* file system id */
	long    f_namelen;  /* maximum length of filenames */
	long    f_spare[6]; /* spare for later */
} statfs_t;


/*****************************************************************************
 *****************************************************************************/

#pragma pack (pop)

#endif
